<?php
namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Api;

use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\BoniCheck2Request;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Order2Request;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\OrderStornoRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Services\Caster;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Change2Request;
use Mille\PlatformConsorsFinanzRK\Components\Logger\LoggerService;

class ConsorsSoapClient extends \SoapClient
{
    const CIPHERS = 'DHE-RSA-AES256-SHA; DHE-RSA-AES128-SHA; AES256-SHAAES128-SHA';
    const CIPHERS2 = 'ECDHE-RSA-AES256-GCM-SHA384;ECDHE-RSA-AES256-SHA384;DHE-RSA-AES256-GCM-SHA384;DHE-RSA-AES256-SHA256;ECDHE-RSA-AES256-SHA;DHE-RSA-AES256-SHA';

    /**
     * @var string
     */
    private $wsdl;

    /**
     * @var string
     */
    private $cafile;
    /**
     * @var mixed
     */
    private $pluginRoot;

    /**
     * Reference of class caster service
     */
    private $caster;
    /**
     * @var LoggerService
     */
    private $loggerService;

    /**
     * @param LoggerService $loggerService
     * @throws \SoapFault
     */
    public function __construct(LoggerService $loggerService)
    {
        $pluginRootPath = \dirname(__DIR__) . '/../..';
        $this->pluginRoot = $pluginRootPath;
        $this->caster = new Caster();

        $this->setApiFiles();
        parent::__construct($this->wsdl, $this->getApiOptions());
        $this->loggerService = $loggerService;
    }

    /**
     *
     * @param
     *            $method
     * @param BoniCheck2Request $params
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function callClient($method, BoniCheck2Request $params)
    {
        return $this->_call($method, [$params]);
    }

    /**
     *
     * @param BoniCheck2Request $request
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function boniCheck2(BoniCheck2Request $request)
    {
        return $this->_call('BoniCheck2', [$request]);
    }

    /**
     *
     * @param Order2Request $request
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function order2(Order2Request $request)
    {
        return $this->_call('Order2', [$request]);
    }

    /**
     *
     * @param OrderStornoRequest $request
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function cancelOrder(OrderStornoRequest $request)
    {
        return $this->_call('OrderStorno', [$request]);
    }

    /**
     *
     * @param Change2Request $request
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function changeOrder(Change2Request $request)
    {
        return $this->_call('Change2', [$request]);
    }

    /**
     * Make SOAP call and catch faults
     *
     * @param string $function
     * @param array $params
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    private function _call($function, $params)
    {
        try {
            return $this->caster->cast($function.'Response',
                $this->__soapCall($function, $params));

        } catch (ConsorsSoapClientFault $e) {
            $this->loggerService->error($e->getMessage());
            throw new ConsorsSoapClientFault($e->getCode(), $e->getMessage());
        }
    }

    private function getApiOptions()
    {
        $sslContextOptions = [
            'ssl' => [
                'verify_peer' => true,
                'cafile' => $this->cafile,
                'ciphers' => self::CIPHERS2
            ]
        ];

        $sslContext = stream_context_create($sslContextOptions);

        return [
            'stream_context' => $sslContext,
            'soap_version' => SOAP_1_1,
            'trace' => 1,
            'exceptions' => 1
        ];
    }

    private function setApiFiles()
    {
        if (!class_exists('SOAPClient')) {
            throw new \Exception('Dieses Plugin benötigt die PHP-Erweiterung "SOAPClient". Bitte wenden Sie sich an Ihren Hoster bzw. Serveradministrator, um diese zu aktivieren.');
        }

        $this->wsdl = $this->pluginRoot . '/wsdl/service_1_4_1.wsdl';
        $this->cafile = $this->pluginRoot . '/wsdl/00_real_solution_ca_bundle.pem';

        if (!file_exists($this->wsdl)) {
            $this->loggerService->error(sprintf('The wsdl %s is missing', $this->wsdl));
        }

        if (!file_exists($this->cafile)) {
            $this->loggerService->error(sprintf('The bundle %s is missing', $this->cafile));
        }

        ini_set('soap.wsdl_cache_enabled',0);
        ini_set('soap.wsdl_cache_ttl',0);
        libxml_disable_entity_loader(false);
    }
}
