<?php
declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Request;

use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Change\ChangeRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Customer\CustomerRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Order\OrderRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Storno\StornoRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\System\SystemRequest;
use Symfony\Component\DependencyInjection\ContainerInterface;

abstract class AbstractRequestFactory
{
    /** @var CustomerRequest */
    protected $customerRequest;

    /** @var SystemRequest */
    protected $systemRequest;
    /**
     * @var ContainerInterface
     */
    protected $container;
    /**
     * @var OrderRequest
     */
    protected $orderRequest;
    /**
     * @var ChangeRequest
     */
    protected $changeRequest;
    /**
     * @var StornoRequest
     */
    protected $stornoRequest;

    public function __construct(
        CustomerRequest $customerRequest,
        SystemRequest $systemRequest,
        OrderRequest $orderRequest,
        ChangeRequest $changeRequest,
        StornoRequest $stornoRequest,
        ContainerInterface $container
    ) {
        $this->customerRequest        = $customerRequest;
        $this->systemRequest          = $systemRequest;
        $this->container = $container;
        $this->orderRequest = $orderRequest;
        $this->changeRequest = $changeRequest;
        $this->stornoRequest = $stornoRequest;
    }

    private $listOfChars = [
        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M',
        'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
        'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l',
        'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x',
        'y', 'z', '1', '2', '3', '4', '5', '6', '7', '8', '9'
    ];

    /** @var array[] */
    protected $requests;

    protected function createRequest($withUserreference = false)
    {
        $parameters = [];

        foreach ($this->requests as $request) {
            $parameters = array_merge($parameters, $request);
        }

        // Clear requests to allow for multiple calls
        $this->requests = [];

        ksort($parameters, SORT_NATURAL | SORT_FLAG_CASE);

        if (!empty($parameters['userreference'])) {
            return $parameters;
        }

        if ($withUserreference == true) {
            $parameters['userreference'] = $this->getUserReference();
        }

        return $parameters;
    }

    protected function getUserReference()
    {
        $session = $this->container->get('session');
        if (empty($session->get('consorsUserReference'))) {
            $consorsUserReference = $this->getRandomString(9);
            $session->set('consorsUserReference', $consorsUserReference);
        }
        return $session->get('consorsUserReference');
    }


    /**
     * This method generates a random string. The $length indicates
     * the length of the resulting string. The $charsToExclude indicates
     * which of the elements are not used. By default 'l', 'i' and '1' are
     * not used.
     *
     * @param integer $length
     * @param string $charsToExclude
     * @return string Random string
     */
    public function getRandomString($length, $charsToExclude = 'li1' ) {
        $characters = [];
        foreach ($this->listOfChars as $char) {
            if (!strstr($charsToExclude, $char)) {
                $characters[] = $char;
            }
        }

        $numberOfCharacters = count($characters);
        $code = '';
        for ($c = 0; $c < $length; $c++) {
            $randIndex = mt_rand(0, $numberOfCharacters-1);
            $code .= $characters[$randIndex];
        }

        return $code;
    }
}
