<?php declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Request;

use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Change\ChangeManagementRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Change\ChangeRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Customer\CustomerRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Order\OrderRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Storno\StornoRequest;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\System\SystemRequest;
use Mille\PlatformConsorsFinanzRK\Struct\PaymentTransaction;
use Shopware\Core\Checkout\Document\DocumentEntity;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\DependencyInjection\ContainerInterface;

class AuthorizeRequestFactory extends AbstractRequestFactory
{
    /**
     * @var ChangeManagementRequest
     */
    private $changeManagementRequest;

    public function __construct(
        CustomerRequest $customerRequest,
        SystemRequest $systemRequest,
        OrderRequest $orderRequest,
        ChangeRequest $changeRequest,
        StornoRequest $stornoRequest,
        ContainerInterface $container,
        ChangeManagementRequest $changeManagementRequest
    ) {
        parent::__construct($customerRequest, $systemRequest, $orderRequest, $changeRequest, $stornoRequest,
            $container);
        $this->changeManagementRequest = $changeManagementRequest;
    }

    public function getRequestParameters(
        PaymentTransaction $transaction,
        RequestDataBag $dataBag,
        SalesChannelContext $context,
        string $anfrageart,
        string $type = 'boni2'
    ) {
        $this->requests[] = $this->systemRequest->getRequestParameters(
            $transaction->getOrder()->getSalesChannelId(),
            $anfrageart
        );

        $this->requests[] = $this->customerRequest->getRequestParameters(
            $dataBag,
            $anfrageart,
            $context
        );

        $this->requests[] = $this->orderRequest->getRequestParameters(
            $transaction,
            $type
        );

        switch ($type) {
            case 'boni2':
                return new BoniCheck2Request($this->createRequest(true));
            case 'order2':
                return new Order2Request($this->createRequest(true));
        }
    }

    public function getChangeRequestParameters(
        DocumentEntity $document,
        string $changetyp,
        bool $isGutschrift = false
    ) {
        $this->requests[] = $this->systemRequest->getRequestParameters(
            $document->getOrder()->getSalesChannelId(),
            $changetyp
        );

        $this->requests[] = $this->changeRequest->getRequestParameters(
            $document,
            $isGutschrift
        );

        return new Change2Request($this->createRequest());
    }

    public function getChangeManagementRequestParameters(
        OrderEntity $order,
        string $changetyp,
        bool $isGutschrift = false
    ) {
        $this->requests[] = $this->systemRequest->getRequestParameters(
            $order->getSalesChannelId(),
            $changetyp
        );

        $this->requests[] = $this->changeManagementRequest->getParameters(
            $order,
            $isGutschrift
        );

        return new Change2Request($this->createRequest());
    }

    public function getStornoRequestParameters(
        OrderEntity $order,
        string $changetyp,
        $referenzwert = ''
    ) {
        $this->requests[] = $this->systemRequest->getRequestParameters(
            $order->getSalesChannelId(),
            $changetyp
        );

        $this->requests[] = $this->stornoRequest->getRequestParameters(
            $order,
            $referenzwert
        );

        return new OrderStornoRequest($this->createRequest());
    }
}
