<?php declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Change;

use Mille\PlatformConsorsFinanzRK\Bootstrap\CustomFieldInstaller;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzDebit;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionCollection;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\OrderEntity;

class ChangeManagementRequest extends ChangeRequest
{
    public function getParameters(OrderEntity $orderEntity, bool $isGutschrift = false): array
    {
        $belege = $this->getBelege($orderEntity, $isGutschrift);
        $debitor = $this->getDebitor($orderEntity);

        return [
            'referenzen' => [
                'referenztyp' => 'bestellnummer',
                'referenzwert' => $orderEntity->getOrderNumber(),
            ],
            'debitor' => $debitor,
            'belege' => $belege,
        ];
    }

    private function getBelege(OrderEntity $orderEntity, $isGutschrift = false)
    {
        $customer = $orderEntity->getOrderCustomer()->getCustomer();
        $customFields = $orderEntity->getCustomFields();
        $documentNumber = $customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_INVOICE_NUMBER];

        /** @var null|OrderTransactionCollection $transactions */
        $transactions = $orderEntity->getTransactions();

        /** @var OrderTransactionEntity $orderTransaction */
        $orderTransaction = $transactions->first();

        $paymentMethod = $orderTransaction->getPaymentMethod();

        $belegreferenz = $customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_INVOICE_REFERENCE];
//        $tax = $this->getTaxFromGrossAndNet($orderEntity->getAmountTotal(), $orderEntity->getAmountNet());
        $tax = $orderEntity->getPrice()->getCalculatedTaxes()->first()->getTaxRate();

        if ('' == $belegreferenz) {
            $belegreferenz = $documentNumber;
        }

        $betrag = round($customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_AMOUNT] * 100, 2);
        $betragNetto = round($this->getNetto($customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_AMOUNT], $tax) * 100, 2);

        $beleg = [
            'kundennummer' => $customFields[CustomFieldInstaller::CONSORS_FINANZ_CUSTOMER_NUMBER] ?? $customer->getCustomerNumber(),
            'belegtyp' => $customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_INVOICE_TYPE],
            'belegnummer' => $documentNumber,
            'belegdatum' => $this->convertDate($customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_DATE]),
            'faellig' => $this->convertDate($customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_DUE_DATE]),
            'waehrung' => $orderEntity->getCurrency()->getIsoCode(),
            'betrag' => $isGutschrift ? $betrag * -1 : $betrag,
            'betrag_netto' => $isGutschrift ? $betragNetto * -1 : $betragNetto,
            'zahlart' => (ConsorsFinanzDebit::NAME == $paymentMethod->getName()) ? 'LA' : 'UEB',
            'rgbezahlt' => 'NEIN',
            'belegpositionen' => $this->getBelegPositions($orderEntity, true),
        ];

        if ($isGutschrift) {
            $beleg['belegreferenz'] = $belegreferenz;
            $beleg['belegtext'] = 'Gutschrift zu Rechnung ' . $belegreferenz . ' vom ' . $orderEntity->getCreatedAt()->format('d.m.Y');
        } else {
            $beleg['belegtext'] = 'Rechnung zur Bestellung ' . $orderEntity->getOrderNumber() . ' vom ' . $orderEntity->getCreatedAt()->format('d.m.Y');
        }

        return $beleg;
    }

    private function getDebitor(OrderEntity $orderEntity)
    {
        $customer = $orderEntity->getOrderCustomer()->getCustomer();
        $orderBillingAddress = $orderEntity->getAddresses()->get($orderEntity->getBillingAddressId());
        $customFields = $orderEntity->getCustomFields();

        return [
            'kundennummer' => $customFields[CustomFieldInstaller::CONSORS_FINANZ_CUSTOMER_NUMBER] ?? $customer->getCustomerNumber(),
            'bezeichnung' => $customer->getFirstName() . ' ' . $customer->getLastName(),
            'person' => $this->getPerson($customer, $orderBillingAddress)
        ];
    }

    protected function convertDate($originalDate)
    {
        if ('' == $originalDate) {
            return '';
        }
        return date("Y-m-d", strtotime($originalDate));
    }

    protected function getTaxFromGrossAndNet($gross, $net)
    {
        return round((($gross - $net) / $net) * 100, 2);
    }

    function dumb($data)
    {
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}
