<?php declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Request;

use Mille\PlatformConsorsFinanzRK\Struct\PaymentTransaction;
use Shopware\Core\Checkout\Document\DocumentEntity;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;

class ManagementRequestFactory extends AbstractRequestFactory
{
    public function getRequestParameters(
        PaymentTransaction $transaction,
        RequestDataBag $dataBag,
        SalesChannelContext $context,
        string $anfrageart,
        string $type = 'boni2'
    ) {
        $this->requests[] = $this->systemRequest->getRequestParameters(
            $transaction->getOrder()->getSalesChannelId(),
            $anfrageart
        );

        $this->requests[] = $this->customerRequest->getRequestParameters(
            $dataBag,
            $anfrageart,
            $context
        );

        $this->requests[] = $this->orderRequest->getRequestParameters(
            $transaction,
            $type
        );

        switch ($type) {
            case 'boni2':
                return new BoniCheck2Request($this->createRequest(true));
            case 'order2':
                return new Order2Request($this->createRequest(true));
        }
    }

    public function getChangeRequestParameters(
        DocumentEntity $document,
        string $changetyp,
        bool $isGutschrift = false
    ) {
        $this->requests[] = $this->systemRequest->getRequestParameters(
            $document->getOrder()->getSalesChannelId(),
            $changetyp
        );

        $this->requests[] = $this->changeRequest->getRequestParameters(
            $document,
            $isGutschrift
        );

        return new Change2Request($this->createRequest());
    }

    public function getStornoRequestParameters(
        OrderEntity $order,
        string $changetyp
    ) {
        $this->requests[] = $this->systemRequest->getRequestParameters(
            $order->getSalesChannelId(),
            $changetyp
        );

        $this->requests[] = $this->stornoRequest->getRequestParameters(
            $order
        );

        return new OrderStornoRequest($this->createRequest());
    }
}
