<?php

namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Services;

use stdClass;

class Caster
{
    const COMPONENT_NAMESPACE = '\Mille\PlatformConsorsFinanzRK\Components\Consors';

    /**
     * @var array $classMap - Map of defined Consors classes
     */
    private $classMap = array(
        'BoniCheck2Request' => self::COMPONENT_NAMESPACE.'\Request\BoniCheck2Request',
        'BoniCheck2Response' => self::COMPONENT_NAMESPACE.'\Response\BoniCheck2Response',
        'Order2Request' => self::COMPONENT_NAMESPACE.'\Request\Order2Request',
        'Order2Response' => self::COMPONENT_NAMESPACE.'\Response\Order2Response',
        'OrderStornoRequest' => self::COMPONENT_NAMESPACE.'\Request\OrderStornoRequest',
        'OrderStornoResponse' => self::COMPONENT_NAMESPACE.'\Response\OrderStornoResponse',
        'Change2Request' => self::COMPONENT_NAMESPACE.'\Request\Change2Request',
        'Change2Response' => self::COMPONENT_NAMESPACE.'\Response\Change2Response',
    );

    /**
     *
     * @param $className
     * @param stdClass $object
     * @return mixed
     */
    public function cast($className, stdClass $object)
    {
        $className = ucfirst($className);
        if (!isset($this->classMap[$className])) {
            throw new \InvalidArgumentException('Class is not castable in Consors namespace!', 666);
        }

        $targetClass = $this->classMap[$className];
        $new = new $targetClass();

        foreach($object as $property => $value) {
            if (method_exists($new, ($methodName = 'set'.ucfirst($property)))) {
                $new->{'set'.ucfirst($property)}($value);
            }
        }
        return $new;
    }

    /**
     * Array wrapper for IMT class casting
     *
     * @param $className
     * @param $objectArr
     * @return array
     */
    public function castArray($className, $objectArr)
    {
        $newArr = [];
        foreach($objectArr as $key => $object) {
            $newArr[$key] = $this->cast($className, $object);
        }
        return $newArr;
    }
}