<?php
namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Services;

use Mille\PlatformConsorsFinanzRK\Bootstrap\CustomFieldInstaller;
use Mille\PlatformConsorsFinanzRK\Components\SettingsTable;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\ConsorsFinanzDebitPaymentHandler;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\ConsorsFinanzInvoicePaymentHandler;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzDebit;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzInvoice;
use Shopware\Core\Checkout\Document\DocumentEntity;
use Shopware\Core\Checkout\Document\DocumentGenerator\CreditNoteGenerator;
use Shopware\Core\Checkout\Document\DocumentGenerator\InvoiceGenerator;
use Shopware\Core\Checkout\Document\DocumentGenerator\StornoGenerator;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionCollection;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStates;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Payment\PaymentMethodEntity;
use Shopware\Core\Framework\Api\Context\SystemSource;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\MultiFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\NotFilter;

class CronService
{
    /**
     * @var EntityRepositoryInterface
     */
    private $documentRepository;
    /**
     * @var EntityRepositoryInterface
     */
    private $orderRepository;
    /**
     * @var EntityRepositoryInterface
     */
    private $paymentMethodRepository;
    /**
     * @var ChangeService
     */
    private $changeService;
    /**
     * @var OrderStornoService
     */
    private $orderStornoService;

    /**
     * @var EntityRepositoryInterface
     */
    private $stateMachineRepository;
    /**
     * @var EntityRepositoryInterface
     */
    private $stateMachineStateRepository;
    /**
     * @var ChangeManagementSystemService
     */
    private $changeManagementSystemService;

    /**
     * CronService constructor.
     * @param EntityRepositoryInterface     $documentRepository
     * @param EntityRepositoryInterface     $orderRepository
     * @param EntityRepositoryInterface     $paymentMethodRepository
     * @param EntityRepositoryInterface     $stateMachineRepository
     * @param EntityRepositoryInterface     $stateMachineStateRepository
     * @param ChangeService                 $changeService
     * @param OrderStornoService            $orderStornoService
     * @param ChangeManagementSystemService $changeManagementSystemService
     */
    public function __construct(
        EntityRepositoryInterface $documentRepository,
        EntityRepositoryInterface $orderRepository,
        EntityRepositoryInterface $paymentMethodRepository,
        EntityRepositoryInterface $stateMachineRepository,
        EntityRepositoryInterface $stateMachineStateRepository,
        ChangeService $changeService,
        OrderStornoService $orderStornoService,
        ChangeManagementSystemService $changeManagementSystemService
    ) {
        $this->documentRepository = $documentRepository;
        $this->orderRepository = $orderRepository;
        $this->paymentMethodRepository = $paymentMethodRepository;
        $this->changeService = $changeService;
        $this->orderStornoService = $orderStornoService;
        $this->stateMachineRepository = $stateMachineRepository;
        $this->stateMachineStateRepository = $stateMachineStateRepository;
        $this->changeManagementSystemService = $changeManagementSystemService;
    }

    public function handleManagementSystem(DataModel $model, $salesChannelId)
    {
        $context = $model->getContext();
        $wConsorsFinanzPaymentIds = $this->getConsorsPaymentMethods($context);

        $orders = $this->getConsorsOrders($model, $salesChannelId);

        $counter = 0;
        /** @var OrderEntity $order */
        foreach ($orders as $order) {
            $paymentId = $order->get('transactions')->first()->get('paymentMethodId');
            $customFields = $order->getCustomFields();
            if (empty($customFields) || empty($order) || !in_array($paymentId, $wConsorsFinanzPaymentIds)) {
                continue;
            }

            /** @var null|OrderTransactionCollection $transactions */
            $transactions = $order->get('transactions');

            /** @var OrderTransactionEntity $orderTransaction */
            $orderTransaction = $transactions->first();

            if (null == $transactions->first()) {
                continue;
            }
            $paymentMethod = $transactions->first()->get('paymentMethod');

            if (false === (bool)$customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_CHANGE_FLAG]) {
                continue;
            }

            switch ($customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_INVOICE_TYPE]) {
                case 'RG':
                case 'TR':
                    // Buchung
                    $orderChangeResponse = $this->changeManagementSystemService->change($order, SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK);
                    if (0 != $orderChangeResponse->getFehlergrund()) {
                        $model->log($orderChangeResponse->getFehlertext() . ' (' . $order->getOrderNumber() . ')', [
                            'order' => $order->getId(),
                            'transactionId' => $orderTransaction->getId(),
                            'orderNumber' => $order->getOrderNumber()
                        ]);
                    } else {
                        $model->log(sprintf('Order %s successfully changed', $order->getOrderNumber()), [
                            'order' => $order->getId(),
                            'transactionId' => $orderTransaction->getId(),
                            'orderNumber' => $order->getOrderNumber()
                        ]);
                        // here book
                        $this->orderRepository->upsert([[
                            'id' => $order->getId(),
                            'customFields' => [
                                CustomFieldInstaller::CONSORS_FINANZ_ORDER_CHANGE_FLAG => false
                            ]
                        ]], $context);

                        ++$counter;
                    }
                    break;
                case 'ST':
                    if (isset($customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_INVOICE_REFERENCE]) && $customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_INVOICE_REFERENCE]) {

                        $requestType = '';
                        if (ConsorsFinanzInvoice::NAME == $paymentMethod->getName()) {
                            $requestType = SettingsTable::REQUEST_TYPE_INVOICE;
                        } elseif (ConsorsFinanzDebit::NAME == $paymentMethod->getName())  {
                            $requestType = SettingsTable::REQUEST_TYPE_DEBIT;
                        }

                        if ('' != $requestType) {
                            $orderStornoResponse = $this->orderStornoService->cancel($order, $requestType, $customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_INVOICE_REFERENCE]);
                            if (0 != $orderStornoResponse->getFehlergrund()) {
                                $model->log($orderStornoResponse->getFehlertext() . ' (' . $order->getOrderNumber() . ')',
                                    [
                                        'order' => $order->getId(),
                                        'transactionId' => $orderTransaction->getId(),
                                        'orderNumber' => $order->getOrderNumber()
                                    ]);
                            } else {
                                $model->log(sprintf('Order %s successfully cancelled',
                                    $order->getOrderNumber()), [
                                    'order' => $order->getId(),
                                    'transactionId' => $orderTransaction->getId(),
                                    'orderNumber' => $order->getOrderNumber()
                                ]);
                                $this->orderRepository->upsert([
                                    [
                                        'id' => $order->getId(),
                                        'customFields' => [
                                            CustomFieldInstaller::CONSORS_FINANZ_ORDER_CHANGE_FLAG => false
                                        ],
                                    ],
                                ], $context);

                                ++$counter;
                            }
                        }
                    }
                    break;
                case 'GU':
                    // Gutschrift
                    $orderChangeResponse = $this->changeManagementSystemService->change($order, SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK, true);
                    if (0 != $orderChangeResponse->getFehlergrund()) {
                        $model->log($orderChangeResponse->getFehlertext() . ' (' . $order->getOrderNumber() . ')', [
                            'order' => $order->getId(),
                            'transactionId' => $orderTransaction->getId(),
                            'orderNumber' => $order->getOrderNumber()
                        ]);
                    } else {
                        $model->log(sprintf('Order %s successfully changed', $order->getOrderNumber()), [
                            'order' => $order->getId(),
                            'transactionId' => $orderTransaction->getId(),
                            'orderNumber' => $order->getOrderNumber()
                        ]);

                        $this->orderRepository->upsert([
                            [
                                'id' => $order->getId(),
                                'customFields' => [
                                    CustomFieldInstaller::CONSORS_FINANZ_ORDER_CHANGE_FLAG => false
                                ],
                            ],
                        ], $context);
                        ++$counter;
                    }
                    break;
            }
        }

        return $counter;
    }

    public function handleShopwareSystem(DataModel $model, $salesChannelId)
    {
        $context = $model->getContext();
        $wConsorsFinanzPaymentIds = $this->getConsorsPaymentMethods($context);

        $documents = $this->getOderDocuments($model, $salesChannelId);

        $counter = 0;
        /** @var DocumentEntity $document */
        foreach ($documents as $document) {
            $order = $document->getOrder();

            $paymentId = $document->getOrder()->get('transactions')->first()->get('paymentMethodId');

            $documentType = $document->getDocumentType();
            if (empty($order) || !in_array($paymentId, $wConsorsFinanzPaymentIds)) {
                continue;
            }

            /** @var null|OrderTransactionCollection $transactions */
            $transactions = $order->get('transactions');

            /** @var OrderTransactionEntity $orderTransaction */
            $orderTransaction = $transactions->first();

            if (null == $transactions->first()) {
                continue;
            }
            $paymentMethod = $transactions->first()->get('paymentMethod');
            $customFields = $order->getCustomFields();

            switch ($documentType->getTechnicalName()) {
                case InvoiceGenerator::INVOICE:
                    if (!isset($customFields[CustomFieldInstaller::CONSORS_FINANZ_IS_ORDER_BOOKED]) || false == (bool)$customFields[CustomFieldInstaller::CONSORS_FINANZ_IS_ORDER_BOOKED]) {
                        // Buchung
                        $orderChangeResponse = $this->changeService->change($document,
                            SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK);
                        if (0 != $orderChangeResponse->getFehlergrund()) {
                            $model->log($orderChangeResponse->getFehlertext() . ' (' . $order->getOrderNumber() . ')',
                                [
                                    'order' => $order->getId(),
                                    'transactionId' => $orderTransaction->getId(),
                                    'orderNumber' => $order->getOrderNumber()
                                ]);
                        } else {
                            $model->log(sprintf('Order %s successfully changed',
                                $order->getOrderNumber()), [
                                'order' => $order->getId(),
                                'transactionId' => $orderTransaction->getId(),
                                'orderNumber' => $order->getOrderNumber()
                            ]);
                            // here book
                            $this->orderRepository->upsert([
                                [
                                    'id' => $order->getId(),
                                    'customFields' => [
                                        CustomFieldInstaller::CONSORS_FINANZ_IS_ORDER_BOOKED => true
                                    ]
                                ]
                            ], new Context(new SystemSource()));

                            ++$counter;
                        }
                    }
                    break;
                case StornoGenerator::STORNO:
                    // if not yet cancelled
                    if (!isset($customFields[CustomFieldInstaller::CONSORS_FINANZ_IS_ORDER_CANCELLED]) || false == (bool)$customFields[CustomFieldInstaller::CONSORS_FINANZ_IS_ORDER_CANCELLED]) {
                        // If the oder has been cancelled, we cancel the order2 as well

                        $requestType = '';
                        if (ConsorsFinanzInvoice::NAME == $paymentMethod->getName()) {
                            $requestType = SettingsTable::REQUEST_TYPE_INVOICE;
                        } elseif (ConsorsFinanzDebit::NAME == $paymentMethod->getName())  {
                            $requestType = SettingsTable::REQUEST_TYPE_DEBIT;
                        }

                        if ('' != $requestType) {
                            $orderStornoResponse = $this->orderStornoService->cancel($order, $requestType);
                            if (0 != $orderStornoResponse->getFehlergrund()) {
                                $model->log($orderStornoResponse->getFehlertext() . ' (' . $order->getOrderNumber() . ')',
                                    [
                                        'order' => $order->getId(),
                                        'transactionId' => $orderTransaction->getId(),
                                        'orderNumber' => $order->getOrderNumber()
                                    ]);
                            } else {
                                $model->log(sprintf('Order %s successfully cancelled',
                                    $order->getOrderNumber()), [
                                    'order' => $order->getId(),
                                    'transactionId' => $orderTransaction->getId(),
                                    'orderNumber' => $order->getOrderNumber()
                                ]);
                                $this->orderRepository->upsert([
                                    [
                                        'id' => $order->getId(),
                                        'customFields' => [
                                            CustomFieldInstaller::CONSORS_FINANZ_IS_ORDER_CANCELLED => true
                                        ],
                                    ],
                                ], $context);

                                ++$counter;
                            }
                        }
                    }
                    break;
                case CreditNoteGenerator::CREDIT_NOTE:
                    $creditNumbers = $customFields[CustomFieldInstaller::CONSORS_FINANZ_ORDER_CREDIT_NUMBERS];
                    $arrayCreditNumbers = explode(',', $creditNumbers);
                    if (!in_array($document->getConfig()['documentNumber'], $arrayCreditNumbers)) {
                        // Gutschrift
                        $orderChangeResponse = $this->changeService->change($document,
                            SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK, true);
                        if (0 != $orderChangeResponse->getFehlergrund()) {
                            $model->log($orderChangeResponse->getFehlertext() . ' (' . $order->getOrderNumber() . ')', [
                                'order' => $order->getId(),
                                'transactionId' => $orderTransaction->getId(),
                                'orderNumber' => $order->getOrderNumber()
                            ]);
                        } else {
                            $model->log(sprintf('Order %s successfully changed', $order->getOrderNumber()), [
                                'order' => $order->getId(),
                                'transactionId' => $orderTransaction->getId(),
                                'orderNumber' => $order->getOrderNumber()
                            ]);

                            $newNumbers = '' == $creditNumbers ? $document->getConfig()['documentNumber'] : $creditNumbers . ',' . $document->getConfig()['documentNumber'];

                            $this->orderRepository->upsert([
                                [
                                    'id' => $order->getId(),
                                    'customFields' => [
                                        CustomFieldInstaller::CONSORS_FINANZ_ORDER_CREDIT_NUMBERS => $newNumbers
                                    ],
                                ],
                            ], $context);
                            ++$counter;
                        }
                    }
                    break;
            }
        }

        return $counter;
    }

    private function getOderDocuments(DataModel $model, string $salesChannelId)
    {
        $context = $model->getContext();
        $wConsorsFinanzPaymentIds = $this->getConsorsPaymentMethods($context);

        $cancelledState = null;
        $criteriaSM = new Criteria;
        $criteriaSM->addFilter(new EqualsFilter('technicalName', OrderTransactionStates::STATE_MACHINE));
        $stateMachine = $this->stateMachineRepository->search($criteriaSM, $context)->first();

        if (!empty($stateMachine)) {
            $criteriaSMS = new Criteria();
            $criteriaSMS->addFilter(new EqualsFilter('technicalName', OrderTransactionStates::STATE_CANCELLED));
            $criteriaSMS->addFilter(new EqualsFilter('stateMachineId', $stateMachine->get('id')));
            $cancelledState = $this->stateMachineStateRepository->search($criteriaSMS, $context)->first();
        }

        $criteria = new Criteria();
        $criteria->addAssociation('documentType');
        $criteria->addAssociation('order');
        $criteria->addAssociation('order.orderCustomer.customer');
        $criteria->addAssociation('order.lineItems');
        $criteria->addAssociation('order.deliveries');
        $criteria->addAssociation('order.deliveries.shippingMethod');
        $criteria->addAssociation('order.lineItems.product');
        $criteria->addAssociation('order.lineItems.product.mainVariant');
        $criteria->addAssociation('order.currency');
        $criteria->addAssociation('order.transactions');
        $criteria->addAssociation('order.transactions.paymentMethod');
        $criteria->addAssociation('order.transactions.stateMachineState');
        $criteria->addAssociation('stateMachineState.stateMachine');
        $criteria->addAssociation('order.customFields');
        $criteria->addAssociation('order.addresses');
        $criteria->addAssociation('order.addresses.salutation');
        $criteria->addAssociation('order.addresses.country');

        $criteria->addFilter(new MultiFilter(
            MultiFilter::CONNECTION_OR,
            [
                new EqualsFilter('order.transactions.paymentMethod.id', $wConsorsFinanzPaymentIds[0]),
                new EqualsFilter('order.transactions.paymentMethod.id', $wConsorsFinanzPaymentIds[1]),
            ]
        ));

        $criteria->addFilter(new MultiFilter(
            MultiFilter::CONNECTION_AND,
            [
//                new NotFilter(
//                    NotFilter::CONNECTION_AND,
//                    [new EqualsFilter('order.transactions.stateMachineState.technicalName', OrderTransactionStates::STATE_CANCELLED)]
//                ),
                new NotFilter(
                    NotFilter::CONNECTION_AND,
                    [
                        new EqualsFilter('order.transactions.stateMachineState.stateMachineId', $model->getSettings()['conditionOrderState'][0])
                    ]
                )
            ]
        ));

        $criteria->addFilter(new EqualsFilter('order.salesChannelId', $salesChannelId));

        return $this->documentRepository->search($criteria, $context);
    }

    private function getConsorsOrders(DataModel $model, string $salesChannelId)
    {
        $context = $model->getContext();
        $wConsorsFinanzPaymentIds = $this->getConsorsPaymentMethods($context);

        $criteria = new Criteria();
        $criteria->addAssociation('order');
        $criteria->addAssociation('orderCustomer.customer');
        $criteria->addAssociation('lineItems');
        $criteria->addAssociation('lineItems.product');
        $criteria->addAssociation('lineItems.product.mainVariant');
        $criteria->addAssociation('deliveries');
        $criteria->addAssociation('deliveries.shippingMethod');
        $criteria->addAssociation('currency');
        $criteria->addAssociation('transactions');
        $criteria->addAssociation('transactions.paymentMethod');
        $criteria->addAssociation('transactions.stateMachineState');
        $criteria->addAssociation('stateMachineState.stateMachine');
        $criteria->addAssociation('customFields');
        $criteria->addAssociation('addresses');
        $criteria->addAssociation('addresses.salutation');
        $criteria->addAssociation('addresses.country');

        $criteria->addFilter(new MultiFilter(
            MultiFilter::CONNECTION_OR,
            [
                new EqualsFilter('order.transactions.paymentMethod.id', $wConsorsFinanzPaymentIds[0]),
                new EqualsFilter('order.transactions.paymentMethod.id', $wConsorsFinanzPaymentIds[1]),
            ]
        ));

        $criteria->addFilter(new NotFilter(
            NotFilter::CONNECTION_AND,
            [
                new EqualsFilter('stateMachineState.stateMachineId', $model->getSettings()['conditionOrderStatus'][0])
            ]
        ));

        $criteria->addFilter(new EqualsFilter('order.customFields.consors_finanz_rk_order_change_flag', 1));
        $criteria->addFilter(new EqualsFilter('order.salesChannelId', $salesChannelId));

        return $this->orderRepository->search($criteria, $context);
    }

    private function getConsorsPaymentMethods($context)
    {
        return [$this->getInvoicePaymentMethodId($context), $this->getDebitPaymentMethodId($context)];
    }

    private function getDebitPaymentMethodId($context)
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('handlerIdentifier', ConsorsFinanzDebitPaymentHandler::class));

        /** @var PaymentMethodEntity $debitPayment */
        $debitPayment = $this->paymentMethodRepository
            ->search($criteria, $context)
            ->first();

        return $debitPayment ? $debitPayment->getId() : '';
    }

    private function getInvoicePaymentMethodId($context)
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('handlerIdentifier', ConsorsFinanzInvoicePaymentHandler::class));

        /** @var PaymentMethodEntity $invoicePayment */
        $invoicePayment = $this->paymentMethodRepository
            ->search($criteria, $context)
            ->first();

        return $invoicePayment ? $invoicePayment->getId() : '';
    }

    function dumb($data)
    {
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}