<?php declare(strict_types=1);
namespace Mille\PlatformConsorsFinanzRK\EventListener;

use Mille\PlatformConsorsFinanzRK\PaymentHandler\AbstractPaymentHandler;
use Shopware\Core\Framework\Validation\BuildValidationEvent;
use Shopware\Core\Framework\Validation\DataValidationDefinition;
use Shopware\Core\PlatformRequest;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class CheckoutValidationSubscriber implements EventSubscriberInterface
{
    /** @var RequestStack */
    private $requestStack;

    /** @var ContainerInterface */
    private $container;

    public function __construct(
        RequestStack $requestStack,
        ContainerInterface $container
    ) {
        $this->requestStack = $requestStack;
        $this->container = $container;
    }

    public static function getSubscribedEvents(): array
    {
        return [
            'framework.validation.order.create' => 'validateOrderData',
        ];
    }

    public function validateOrderData(BuildValidationEvent $event): void
    {
        $request = $this->requestStack->getCurrentRequest();

        if (null === $request) {
            return;
        }

        $context = $this->getContextFromRequest($request);
        $paymentHandlerIdentifier = $context->getPaymentMethod()->getHandlerIdentifier();

        if (strpos($paymentHandlerIdentifier, 'PlatformConsorsFinanzRK') !== false) {
            if ($this->container->has($paymentHandlerIdentifier)) {
                /** @var $paymentHandler AbstractPaymentHandler */
                $paymentHandler = $this->container->get($paymentHandlerIdentifier);

                $validationDefinitions = $paymentHandler->getValidationDefinitions($request, $context);

                $definitions = new DataValidationDefinition();
                $this->addSubConstraints($definitions, $validationDefinitions);
                $event->getDefinition()->addSub('consorsfinanz', $definitions);
            }
        }
    }

    private function getContextFromRequest($request): SalesChannelContext
    {
        return $request->attributes->get(PlatformRequest::ATTRIBUTE_SALES_CHANNEL_CONTEXT_OBJECT);
    }

    protected function addSubConstraints(DataValidationDefinition $parent, array $children)
    {
        foreach ($children as $key => $constraints) {
            if ($constraints instanceof DataValidationDefinition) {
                $parent->addSub($key, $constraints);
            } elseif (is_array($constraints)) {
                call_user_func_array([$parent, 'add'], array_merge([$key], $constraints));
            } else {
                continue;
            }
        }
    }
}
