<?php declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\ScheduledTask;

use Mille\PlatformConsorsFinanzRK\Components\Consors\Services\CronService;
use Mille\PlatformConsorsFinanzRK\Components\Factory\ModelFactoryInterface;
use Shopware\Core\Framework\Api\Context\SystemSource;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\MessageQueue\ScheduledTask\ScheduledTaskHandler;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\System\SalesChannel\SalesChannelEntity;

class OrderChangeHandler extends ScheduledTaskHandler
{
    /**
     * @var EntityRepositoryInterface
     */
    private $salesChannelRepository;
    /**
     * @var ModelFactoryInterface
     */
    private $modelFactory;
    /**
     * @var CronService
     */
    private $cronService;

    public function __construct(
            EntityRepositoryInterface $scheduledTaskRepository,
            EntityRepositoryInterface $salesChannelRepository,
            ModelFactoryInterface $modelFactory,
            CronService $cronService
        ) {
        parent::__construct($scheduledTaskRepository);

        $this->salesChannelRepository = $salesChannelRepository;
        $this->modelFactory = $modelFactory;
        $this->cronService = $cronService;
    }

    public static function getHandledMessages(): iterable
    {
        return [ OrderChange::class ];
    }

    public function run(): void
    {
        $output = PHP_EOL;
        $occurredErrors = PHP_EOL;
        $counter = 0;
        try {
            $criteria = new Criteria();
            $criteria->addAssociation('salesChannel');
            $context = new Context(new SystemSource());

            $salesChannels = $this->salesChannelRepository->search($criteria, $context);

            /** @var SalesChannelEntity $salesChannel */
            foreach ($salesChannels as $salesChannel) {
                if (false === (bool)$salesChannel->getActive()) {
                    continue;
                }

                $model = $this->modelFactory->get($salesChannel->getId());
                if (!$model->isActive()) {
                    continue;
                }

                $settings = $model->getSettings();

                if ('no' == $settings['managementSystem']) {
                    $counter = $this->cronService->handleShopwareSystem($model, $salesChannel->getId());
                } else {
                    $counter = $this->cronService->handleManagementSystem($model, $salesChannel->getId());
                }
            }

            $output .= sprintf(
                '%sMilleConsorsFinanzRK: %s order-change processed. %s %s',
                $this->getNewLine(),
                $counter,
                $this->getNewLine(),
                $occurredErrors
            );

            echo $output;
        } catch (\Throwable $e) {
            // catch exception - otherwise the task will never be called again
//            $model->log($e->getMessage(), []);
            return;
        }
    }

    private function getNewLine()
    {
        return PHP_SAPI == 'cli' ? "\n" : '<br>';
    }
}
